
#include	"extern.h"

FPPA			=>	1;
FPPA_Duty		=>	_Get_Duty(Program_Mode, FPPA);

/*	About Syntax	:	_Get_Duty (x, y)

	Example	:					FPPA0	FPPA1	FPPA2	FPPA3	...
				PMODE	0	:	/2		/2
				PMODE	..	:	..		..
				PMODE	30	:	/16		/2		/8		/16		...

	so, if PMODE = 0, at FPPA0, then duty = /2,
	then	_Get_Duty (0, 0)	=	2;

	follow is PMODE = 30, at different FPPAx,
  	_Get_Duty (30, 0)	=	16;  	_Get_Duty (30, 1)	=	2;
  	_Get_Duty (30, 2)	=	8;  	_Get_Duty (30, 3)	=	16;
*/

#if	FPPA_Duty == 0
	.ERROR	FPPA_Duty is 0
	.ENDP
#endif

Baud_Rate		=>	38400;
/*
UART_Delay		=>	(System_Clock / FPPA_Duty) / Baud_Rate;

	if	System_Clock	=	8,000.000 Hz
		FPPA_Duty		=	/16
	so	FPPA_Clock		=	System_Clock / FPPA_Duty	=	500.000	Hz

	if	Baud_Rate		=	19200
	so	UART_Delay		=	500.000 / 19200 = 26.0416...
	so	match, 26 cycles send one bit.	< 0.5%

	if	Baud_Rate		=	38400
	so	UART_Delay		=	500.000 / 38400 = 13.02083...
	so	match, 13 cycles send one bit.	< 0.5%

	if	Baud_Rate		=	56000
?	so	UART_Delay		=	500.000 / 56000 = 8.9285...		<	1.0%

	if	Baud_Rate		=	57600
X	so	UART_Delay		=	500.000 / 57600 = 8.6805...		:	fail

*/
UART_Delay		=>	( (System_Clock / FPPA_Duty) + (Baud_Rate/2) ) / Baud_Rate;
//	+ (Baud_Rate/2) : to round up or down

Test_V0			=>	System_Clock / 1000 * 995;
Test_V1			=>	UART_Delay * Baud_Rate * FPPA_Duty;
Test_V2			=>	System_Clock / 1000 * 1005;

#if	(Test_V1 < Test_V0) || (Test_V1 > Test_V2)
	.echo	%Test_V0 <= %Test_V1 <= %Test_V2
	.error	Baud_Rate do not match to System Clock
#endif

UART_Out	BIT	PB.0;
UART_In		BIT	PB.1;

void	UART_Send (void)
{
	BYTE	cnt;
	BYTE	UART_Data_Out;

	UART_Data_Out	=	A;

	//	Start Bit
	set0	UART_Out;				//	1

	#if	FPPA_Duty == 1
		cnt	=	8;						//	2 ~ 3
		.Delay	3;						//	4 ~ 6
		do
		{	//	Data Bit * 8
			.Delay	UART_Delay - 10;
			sr		UART_Data_Out;		//	7
			if (CF)
			{
				nop;					//	10
				UART_Out	=	1;		//	1
			}
			else
			{
				UART_Out	=	0;		//	1
				.delay	2;				//	2 ~ 3
			}
		} while (--cnt);				//	4 ~ 6
		.Delay	UART_Delay - 5;
	#else
		.Delay	UART_Delay - 4;
		cnt	=	8;						//	2 ~ 3

		//	Data Bit * 8
		do
		{
			sr		UART_Data_Out;		//	4		4
			swapc	UART_Out;			//			1
			.Delay	UART_Delay - 4;
		} while (--cnt);				//			2, 3

		.Delay	2;						//			3 ~ 4
	#endif

	//	Stop Bit
	set1	UART_Out;				//	1
	.Delay	2 * UART_Delay - 2;
}									//	2

static	void	UART_Receive (void)
{
	while (1)
	{
		BYTE	cnt	=	8, data;

		if (! UART_In)
		{
err:		//	receive UART error, so ...;
			continue;
		}
		//	Wait Start Bit
		while (UART_In) NULL;

		#if	FPPA_Duty == 1
			.Delay	(UART_Delay / 2) -2;
			if (UART_In) goto err;			//	1, 2
			.Delay	UART_Delay - 3;
			CF	=	0;						//	3
			do
			{
				t0sn	UART_In;			//	1
				CF	=	1;					//	2
				src		data;				//	3
				.Delay	UART_Delay - 6;
			} while (--cnt);				//	4 ~ 6
		#else
			.Delay	UART_Delay / 2;
			if (UART_In) goto err;			//	1

			.Delay	UART_Delay - 1;
			do
			{
				swapc	UART_In;			//	1
				src		data				//	2
				.Delay	UART_Delay - 4;
			} while (--cnt);				//	3, 4
		#endif

		A	=	data;					//	4
		//	Check Stop Bit
		if (! UART_In) goto err;
		return;
	}
}

UART_In_Size	=>	0x10;
static	BYTE	buf_UART_In[UART_In_Size];
WORD	pnt_UART_Get, pnt_UART_In;

void	Init_UART_In (void)
{
	$	UART_In		In;
	$	UART_Out	High, Out;

	pnt_UART_Get	=	buf_UART_In;
	pnt_UART_In		=	buf_UART_In;
}

void	FPPA1 (void)
{
	while (1)
	{
		UART_Receive();
		*pnt_UART_In	=	A;

		A	=	pnt_UART_In$0 + 1;
 		if (A == &buf_UART_In[UART_In_Size])	A	=	buf_UART_In;
		while (A == pnt_UART_Get$0)				NULL;	//	wait another FPPA ready

		pnt_UART_In$0	=	A;
	}
}

/*
	At Another FPPA, use the follow code ..

	extern	WORD			pnt_UART_Get, pnt_UART_In;
	#define	Is_UART_In()	(pnt_UART_Get$0 != pnt_UART_In$0)
	extern	void			Get_UART_In (void);
	extern	void			UART_Send (void);

	if (Is_UART_In())
	{
		Get_UART_In();	...
	}
*/

void	Get_UART_In (void)
{
	A	=	pnt_UART_Get;
	while (A == pnt_UART_In$0) NULL;

	BYTE	data	=	*pnt_UART_Get;

	A	=	pnt_UART_Get$0 + 1;
 	if (A == &buf_UART_In[UART_In_Size])	A	=	buf_UART_In;
	pnt_UART_Get$0	=	A;

	A	=	data;
}

